// SBox1.cpp: Implementierung der Klasse SBox.
//
//////////////////////////////////////////////////////////////////////

#include "sbox.h"
#include "sgamefield.h"

#define SIZE 0.5f

unsigned int SBox::object = 0;

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SBox::SBox()
{
	boxState = NORMAL;
        position[0] = 0.0f; position[1] = 0.0f;
}

SBox::~SBox()
{

}

void SBox::draw()
{
	glPushMatrix();

	texture->select();

	glTranslatef(position[0],position[1],0.0f);

	//call compiled list
        glCallList(object);

	glPopMatrix();
}

void SBox::init(SBMPTexture *tex)
{
	texture = tex;
}

void SBox::update(float frametime)
{
	/* BOXSTATE-SPECIFIC UPDATES */
	draw();
}

void SBox::moveBy(float x, float y)
{
        position[0]+=x;
        position[1]+=y;
}

/** No descriptions */
bool SBox::touchesBoxes(SDirection pos)
{
        /**
         * LEFT
         **********/

        if (pos == LEFT)
        {
                //is on the most left side
                if (position[0] != 0.0f &&
                    SGamefield::field[int(position[0]) - 1][int(position[1])])
                        return true;
                        
                return false;
        }
           
        /**
         * RIGHT
         **********/
        if (pos == RIGHT)
        {
                //is on the most left side
                if (position[0] != float(FIELD_X) &&
                    SGamefield::field[int(position[0]) + 1][int(position[1])])
                        return true;

                return false;
        }
         

         /**
          * BOTTOM
          *********/

        if (pos == BOTTOM)
        {
                //is on the most left side
                if (position[1] != 0.0f &&
                    SGamefield::field[int(position[0])][int(position[1]) - 1])
                        return true;

                return false;
        }

        return false;
}

void SBox::build()
{
        /** Generate box compiled list **/
        object = glGenLists(1);

        glNewList(object, GL_COMPILE);
                glBegin(GL_QUADS);
                        glColor3f(1.0f,1.0f,1.0f);
        		// Front Face
	        	glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		        glTexCoord2f(0.0f, 0.0f); glVertex3f(-SIZE, -SIZE,  SIZE);	// Point 1 (Front)
        		glTexCoord2f(1.0f, 0.0f); glVertex3f( SIZE, -SIZE,  SIZE);	// Point 2 (Front)
	        	glTexCoord2f(1.0f, 1.0f); glVertex3f( SIZE,  SIZE,  SIZE);	// Point 3 (Front)
		        glTexCoord2f(0.0f, 1.0f); glVertex3f(-SIZE,  SIZE,  SIZE);	// Point 4 (Front)
        		// Back Face

	        	glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
	        	glTexCoord2f(1.0f, 0.0f); glVertex3f(-SIZE, -SIZE, -SIZE);	// Point 1 (Back)
	        	glTexCoord2f(1.0f, 1.0f); glVertex3f(-SIZE,  SIZE, -SIZE);	// Point 2 (Back)
	        	glTexCoord2f(0.0f, 1.0f); glVertex3f( SIZE,  SIZE, -SIZE);	// Point 3 (Back)
	                glTexCoord2f(0.0f, 0.0f); glVertex3f( SIZE, -SIZE, -SIZE);	// Point 4 (Back)
	        	// Top Face
	        	glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
	        	glTexCoord2f(0.0f, 1.0f); glVertex3f(-SIZE,  SIZE, -SIZE);	// Point 1 (Top)
	        	glTexCoord2f(0.0f, 0.0f); glVertex3f(-SIZE,  SIZE,  SIZE);	// Point 2 (Top)
		        glTexCoord2f(1.0f, 0.0f); glVertex3f( SIZE,  SIZE,  SIZE);	// Point 3 (Top)
	        	glTexCoord2f(1.0f, 1.0f); glVertex3f( SIZE,  SIZE, -SIZE);	// Point 4 (Top)
	        	// Bottom Face
	        	glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
	        	glTexCoord2f(1.0f, 1.0f); glVertex3f(-SIZE, -SIZE, -SIZE);	// Point 1 (Bottom)
	        	glTexCoord2f(0.0f, 1.0f); glVertex3f( SIZE, -SIZE, -SIZE);	// Point 2 (Bottom)
		        glTexCoord2f(0.0f, 0.0f); glVertex3f( SIZE, -SIZE,  SIZE);	// Point 3 (Bottom)
	        	glTexCoord2f(1.0f, 0.0f); glVertex3f(-SIZE, -SIZE,  SIZE);	// Point 4 (Bottom)
		        // Right face
		        glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
        		glTexCoord2f(1.0f, 0.0f); glVertex3f( SIZE, -SIZE, -SIZE);	// Point 1 (Right)
	        	glTexCoord2f(1.0f, 1.0f); glVertex3f( SIZE,  SIZE, -SIZE);	// Point 2 (Right)
		        glTexCoord2f(0.0f, 1.0f); glVertex3f( SIZE,  SIZE,  SIZE);	// Point 3 (Right)
        		glTexCoord2f(0.0f, 0.0f); glVertex3f( SIZE, -SIZE,  SIZE);	// Point 4 (Right)
	        	// Left Face
	        	glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
	        	glTexCoord2f(0.0f, 0.0f); glVertex3f(-SIZE, -SIZE, -SIZE);	// Point 1 (Left)
	        	glTexCoord2f(1.0f, 0.0f); glVertex3f(-SIZE, -SIZE,  SIZE);	// Point 2 (Left)
	        	glTexCoord2f(1.0f, 1.0f); glVertex3f(-SIZE,  SIZE,  SIZE);	// Point 3 (Left)
	        	glTexCoord2f(0.0f, 1.0f); glVertex3f(-SIZE,  SIZE, -SIZE);	// Point 4 (Left)
        	glEnd();
        glEndList();
}
