/***************************************************************************
                          shighscorelist.cpp  -  description
                             -------------------
    begin                : Fri Dec 27 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "shighscorelist.h"

#include <algorithm>
#include <fstream.h>
#include <string.h>
#include <stdio.h>

using namespace std;

/** Helper compare function for highscore list **/
inline bool compare(SHCEntry first, SHCEntry second)
{
        return (first.score > second.score);
}     

/** Helper function that translates ' '  into '_' **/
inline char* translateString(char* string)
{
        for (unsigned int i = 0; i < strlen(string); ++i)
        {
                if (string[i] == ' ')
                        string[i] = '_';
        }

        return string;
}

/** Helper function that translates '_' into ' ' **/
inline char* translateStringBack(char* string)
{
        for (unsigned int i = 0; i < strlen(string); ++i)
        {
                if (string[i] == '_')
                        string[i] = ' ';
        }

        return string;
}


SHighscoreList::SHighscoreList()
{
}

SHighscoreList::~SHighscoreList()
{
}

/** No descriptions */
void SHighscoreList::load(const char* filename)
{
        /**
          We have to translate the strings as
          [i|o]fstream always stops at the next space
          and we couldn't use space in names then.
         **/ 
	::ifstream file(filename);
        
        if (file.good())
        {
                for (int i = 0; i < HIGHSCORE_ENTRIES; ++i)
                {
                        file >> entries[i].name >> entries[i].score;
                        strcpy(entries[i].name, translateStringBack(entries[i].name));
                }
        }

        //done.
}

/** No descriptions */
void SHighscoreList::save(const char* filename)
{
        /**
          Check load() for note.
         **/ 
	::ofstream file(filename);

        if (file.good())
        {
                for (int i = 0; i < HIGHSCORE_ENTRIES; ++i)
                {
                        file << translateString(entries[i].name) << endl << entries[i].score << endl;
                }
        }
}

/** No descriptions */
int SHighscoreList::isHighscore(unsigned int score)
{
        //Get position in list by checking whether score
        //is greater than the current one, starting at the top of the
        //list
        for (int maxPos = 0; maxPos < HIGHSCORE_ENTRIES; ++ maxPos)
        {
                if (score > entries[maxPos].score)
                        return maxPos + 1;
                
        }

        return 0;
}

/** No descriptions */
void SHighscoreList::addHighscore(const char* name, unsigned int score)
{
        int size = strlen(name);

        //site mustn't be greater than allowed
        if (size > MAX_NAME_LENGTH)
                size = MAX_NAME_LENGTH;

        strncpy(entries[HIGHSCORE_ENTRIES-1].name,name,MAX_NAME_LENGTH);
        entries[HIGHSCORE_ENTRIES-1].score = score;

        //sort so highscore list is ordered by score with
        //the best highscore besing at entries[0]
        stable_sort(entries, entries + HIGHSCORE_ENTRIES, compare);   
}

/** No descriptions */
void SHighscoreList::draw(int score)
{
        float x = 225.0f,
              y;
        char buffer[64] = {'\0'};
              
        //draw highscore title
        glColor3f(1.0f,0.0f,0.0f);
        font->printAt(300.0f, 350.0f, "Highscores");

        /**
          Draw all highscore items
         **/

        //Go through entries and display place, name
        //and corresponding score.
        for (int i = 0; i < HIGHSCORE_ENTRIES; ++i)
        {
                y = 300.0f-i*20;

                /**
                  If new score, show in red else white
                 **/
                if (entries[i].score == score)
                        glColor3f(1.0f,0.0f,0.0f);
                else
                        glColor3f(1.0f,1.0f,1.0f);

                // Place 
                sprintf(buffer, "%d.", i + 1);
                font->printAt(x, y, buffer);

                // Name 
                font->printAt(x+40,y,entries[i].name);

                // Score 
                sprintf(buffer, "%d", entries[i].score);
                font->printAt(x+240,y, buffer, true);
        }

        /**
          Little key help
         **/

         glColor3f(1.0f,1.0f,1.0f);
         font->printAt(550,10,"ESC",true);
         font->printAt(590,10,"Back");               
}
