/***************************************************************************
                          smenu.cpp  -  description
                             -------------------
    begin                : Sun Dec 29 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "smenu.h"

#define MAIN_ITEMS 5
const char* mainMenuTexts[] = { "Play!", "Options", "Highscores", "Credits", "Exit" };
#define OPTIONS_ITEMS 4
const char* optionsMenuTexts[] = { "Ingame Snowflakes", "Music", "Reflection", "Fullscreen", "Back" };

#define ON_TEXT "On"
#define OFF_TEXT "Off"

SMenu::SMenu()
{
        menu = MENU_MAIN;
        font = 0;
        position = 0;
}

SMenu::~SMenu()
{
}

/** No descriptions */
void SMenu::create(SBMPFont* bitmapFont, SConfiguration* config)
{
        font = bitmapFont;
        configFile = config;
}

/** No descriptions */
void SMenu::moveUp()
{
        position--;

        //Either options or main menu
        if ( menu == MENU_MAIN )
        {
                if (position < 0)
                        position = MAIN_ITEMS-1;
        }
        else if ( menu == MENU_OPTIONS )
        {
                if (position < 0)
                        position = OPTIONS_ITEMS;
        }
}

/** No descriptions */
void SMenu::moveDown()
{
        position++;

        //Either options or main menu
        if ( menu == MENU_MAIN )
        {
                if (position >= MAIN_ITEMS)
                        position = 0;
        }
        else if ( menu == MENU_OPTIONS )
        {
                if (position > OPTIONS_ITEMS)
                        position = 0;
        }
}

/** No descriptions */
SGameState SMenu::enter()
{
        //Either options or main menu
        if ( menu == MENU_MAIN )
        {
                switch (position)
                {
                        case 0:
                                return STATE_GAME;
                        case 1:
                                menu = MENU_OPTIONS;
                                position = 0;
                                return STATE_OPTIONS;
                        case 2:
                                return STATE_HIGHSCORES;
                        case 3:
                                return STATE_CREDITS;
                        case 4:
                                return STATE_END;
                        default:
                                return (SGameState)-1;
                }            
        }
        else if ( menu == MENU_OPTIONS )
        {
                switch (position)
                {
                        case 0:
                                configFile->setSnowFlakes(!configFile->isSnowFlakes());
                                break;
                        case 1:
                                configFile->setMusic(!configFile->isMusic());
                                break;
                        case 2:
                                configFile->setReflection(!configFile->isReflection());
                                break;
                        case 3:
                                configFile->setFullscreen(!configFile->isFullscreen());
                                break;
                        //If last(Back) button, go back to main menu
                        case OPTIONS_ITEMS:
                                back();
                                //signalize window to update itself
                                return STATE_MENU;
                        
                                
                }
                
                //Always no new gamestate as this is internal
                return (SGameState)-1;
           
        }
}

/** No descriptions */
void SMenu::draw()
{
        //Either options or main menu
        if ( menu == MENU_MAIN )
        {
                for (unsigned int i = 0; i < MAIN_ITEMS; ++i)
                {
                        //current will be drawn red
                        if (position == i)
                                glColor3f(1.0f,0.0f,0.0f);
                        else
                                glColor3f(1.0f,1.0f,1.0f);

                        font->printAt(START_X, START_Y - i*(FONT_SIZE+10.0f),
                                mainMenuTexts[i]);
                }
        }
        else if ( menu == MENU_OPTIONS )
        {
                for (unsigned int i = 0; i <= OPTIONS_ITEMS; ++i)
                {
                        //current will be drawn red
                        if (position == i)
                                glColor3f(1.0f,0.0f,0.0f);
                        else
                                glColor3f(1.0f,1.0f,1.0f);

                        //Normal options item or Back button
                        if (i < OPTIONS_ITEMS)
                        {
                                font->printAt(START_X, START_Y - i*(FONT_SIZE+10.0f),
                                        optionsMenuTexts[i]);

                                char* state;
                                
                                //Depending on option, show the current config state
                                switch (i)
                                {
                                case 0:
                                        (configFile->isSnowFlakes() ? state = ON_TEXT : state = OFF_TEXT);
                                        break;
                                case 1:
                                        (configFile->isMusic() ? state = ON_TEXT : state = OFF_TEXT);
                                        break;
                                case 2:
                                        (configFile->isReflection() ? state = ON_TEXT : state = OFF_TEXT);
                                        break;
                                case 3:
                                        (configFile->isFullscreen() ? state = ON_TEXT : state = OFF_TEXT);
                                        break;
                                default:
                                        state = "";
                                }

                                //draw state depending on value
                                font->printAt(START_X + strlen(optionsMenuTexts[i])*10 + 10, START_Y - i*(FONT_SIZE+10.0f),
                                                state, true);                                
                        }
                        else
                                font->printAt(START_X, START_Y - i*(FONT_SIZE+10.0f),
                                        "Back");
                }
                
        }
}
/** No descriptions */
void SMenu::back()
{
        menu = MENU_MAIN;
        position = 0;
}
